/**
 * The Inventory class represents a kobold's inventory and equipment.
 * It contains the amount of coins a kobold is carrying, as well as the kobold's
 * bag of things and the stuff the kobold is currently wearing.
 */
function Inventory() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Creates the body of a tooltip for items.
     * @param {object} item The properly-constructed item.
     * @returns {string} The body of the tooltip.
     */
    function createToolTipBody(item) {
        return item.description();
    }

    /**
     * Creates a tooltip header with the passed name.
     * @param {string} name The name of the item to create a header for.
     * @returns {string} The header as a HTML string.
     */
    function createToolTipHeader(name) {
        return `${name}`;
    }

    /**
     * Creates a footer for a tooltip of the passed consumable.
     * @param {object} consumable The properly-constructed consumable.
     * @returns {string} The footer.
     */
    function createConsumableToolTipFooter(consumable) {
        return `<div class="toolbottomprice">${consumable.price()} ${global.ui.icons.coin()}</div>`;
    }

    /**
     * Creates a footer for a tooltip of the passed piece of gear.
     * @param {object} gear The properly-constructed piece of gear.
     * @returns {string} The footer.
     */
    function createGearToolTipFooter(gear) {
        var footer = `<div class="toolbottomprice">${gear.price()} ${global.ui.icons.coin()}</div>`;
        footer += `<div class="toolbottomdefence">${gear.defence()} ${global.ui.icons.shield()}</div>`;
        footer += `<div class="toolbottomoffence">${gear.offence()} ${global.ui.icons.sword()}</div>`;
        return footer;
    }

    /**
     * Creates a tooltip footer for miscellaneous items.
     * @param {object} misc The properly-constructed miscellaneous item.
     * @returns {string} The tooltip footer.
     */
    function createMiscToolTipFooter(misc) {
        return `<div class="toolbottomprice">${misc.price()} ${global.ui.icons.coin()}</div>`;
    }

    /**
     * Creates a tooltip footer for the passed item.
     * @param {object} item The properly-constructed item.
     * @returns {String} The tooltip footer.
     */
    function createToolTipFooter(item) {
        switch (item.type) {
            case ItemTypeEnum.CONSUMABLE:
                return createConsumableToolTipFooter(item);
            case ItemTypeEnum.GEAR:
                return createGearToolTipFooter(item);
            default:
                return createMiscToolTipFooter(item);
        }
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.bag = new Bag();
    this.equipment = new Equipment();

    // How much money the kobold has.
    this.coins = 0;

    // Whether or not the kobold is in a safe zone (can use consumables).
    this.safeZone = false;
    // Whether or not the kobold is in a sell zone (can sell items).
    this.sellZone = false;
    // Sell price modifier for items. Should always be reset to 1.0 after shops.
    this.sellMod = 1.0;
    // Specific function which can be used to listen to the selling of objects.
    // Must be manually reset to null after usage.
    this.sellListener = null;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds a bold-faced item name which will reveal a tooltip when moused over.
     * @param {object | itemEnum} item The item for which to build a tooltip.
     * @param {boolean} caps Whether or not the item's name should be capitalized. May be omitted.
     * @returns {string} The constructed item name with tooltip.
     */
    this.createItemWithTooltip = function (item, caps) {
        if (!item) {
            return "";
        }
        var mouseOver = `<span class="bold">${item.name(caps)}</span>`;
        return that.createItemTooltip(mouseOver, item);
    };

    /**
     * Builds an item tooltip around the specified mouseover target.
     * @param {string} mouseOver The item which should produce the tooltip on mouseover.
     * @param {object} item The item for which to build a tooltip.
     * @returns {string} The constructed item name with tooltip.
     */
    this.createItemTooltip = function (mouseOver, item) {
        if (!item) {
            return "";
        }
        var body = createToolTipBody(item);
        var header = createToolTipHeader(item.name(true));
        var footer = createToolTipFooter(item);
        var style = item.toolTipStyle();
        return global.ui.makeToolTip(mouseOver, body, header, footer, style);
    };

    /**
     * Gets a list of all items and consumables mixed together in ItemAmount objects.
     * @returns {array} An array of ItemAmount objects representing all consumables and misc items in the kobold's bag.
     */
    this.getItemList = function () {
        var misc = that.bag.getMiscItemsList();
        var cons = that.bag.getConsumablesList();
        var both = misc.concat(cons);
        return both;
    };
    
    /**
     * Adds the passed, properly-constructed item to the kobold's inventory.
     * @param {object} item The properly constructed item.
     */
    this.addItem = function(item) {
        if(item.type === ItemTypeEnum.GEAR){
            that.equipment.wear(item);
        } else if(item.type === ItemTypeEnum.CONSUMABLE) {
            that.bag.addConsumable(item);
        } else {
            that.bag.addMisc(item);
        }
    };

    /**
     * Makes an item out of the passed item id.
     * @param {ItemEnum} itemId The id of the item. 
     * @returns {object} The properly constructed item.
     */
    this.constructItem = function (itemId) {
        return ItemEnum.properties[itemId].create();
    };

    /**
     * Makes an item out of the passed JSONified item stripped of functions.
     * @param {object} jsonItem The item parsed from JSON to an object.
     * @returns {object} The properly constructed item.
     */
    this.constructItemFromMemory = function (jsonItem) {
        if (jsonItem.type === ItemTypeEnum.CONSUMABLE || jsonItem.type === ItemTypeEnum.MISC) {
            return that.constructItem(jsonItem.id);
        } else {
            return that.equipment.buildGearFromHistory(jsonItem);
        }
    };

    /**
     * Properly constructs an array of ItemAmount objects read from memory,
     * ensuring that all of the items within as well as the ItemAmounts themselves are properly initialized.
     * @param {array} list An array of JSONified ItemAmount objects.
     * @returns {array} An array of deeply constructed ItemAmount objects.
     */
    this.constructItemAmountListFromMemory = function (list) {
        var properArray = [];
        for (var i = 0; i < list.length; i++) {
            var jsonElement = list[i];
            var amount = jsonElement.amount;
            var jsonItem = jsonElement.item;
            var item = that.constructItemFromMemory(jsonItem);
            properArray.push(new ItemAmount(amount, item));
        }
        return properArray;
    };

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Inventory-stripped} inventory An Inventory object, stripped of private members and all functions.
     */
    this.load = function (inventory) {
        that.bag.load(inventory.bag);
        that.equipment.load(inventory.equipment);
        that.coins = inventory.coins;
        that.safeZone = inventory.safeZone;
        that.sellZone = inventory.sellZone;
        if (inventory.sellMod) {
            that.sellMod = inventory.sellMod;
        }
    };
}
